#include <ESP8266WiFi.h>
#include "DHT.h"
extern "C" {
#include "user_interface.h"
}

#define DHTPIN 16
#define DHTTYPE DHT11
DHT dht(DHTPIN, DHTTYPE, 15);

const int calibration_time = 30;
const int movepin = 5; // actually - 4
int temp_send_counter = 0;
int moving_detected = 0;
bool last_state = 0;

long loop2Time,currentTime;

const int buttonPin = 14;   // really - 14
const int ledPin = 4;       // actually - 5

bool ledState = HIGH;         // the current state of the output pin
bool led2State = LOW;
bool buttonState;             // the current reading from the input pin
bool lastButtonState = HIGH;
bool countStart = 0;
bool ignoreUp = 0;

#define holdTime 1000
long btnDnTime; // time the button was pressed down

long lastDebounceTime = 0;  // the last time the output pin was toggled
long debounceDelay = 50; 

String sensor_id = "bathroom-fan-switch"; 

const char* ssid     = "net20";
const char* password = "015230015230";

const char* host = "192.168.10.200";

WiFiServer server(80);

void setup() {
    WiFi.mode(WIFI_STA);

  // initialize digital pin 13 as an output.
  pinMode(ledPin, OUTPUT);
  Serial.begin(57600); 
  
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);
  
  WiFi.begin(ssid, password);
//  
//  while (WiFi.status() != WL_CONNECTED) {
//    delay(1000);
//    Serial.print(".");
//  }
  Serial.println("Calibrating sensor...");
  for (int i = 0; i < calibration_time; i++)
    {
      Serial.print(".");
      delay(1000);
    }
    
  Serial.println("");
  Serial.println("DONE. Sensor Active...");
  Serial.println("");
  
  if (WiFi.status() == WL_CONNECTED){
      Serial.println("WiFi connected");  
      Serial.println("IP address: ");
      Serial.println(WiFi.localIP());
  }

  server.begin();
  Serial.println("Server started");
  

  String url = String("/objects/?script=rebooted&name=")+sensor_id;
  send_switch(url);

  
//  digitalWrite(ledPin, 1);
//  pinMode(0, INPUT);
//  pinMode(1, INPUT);
//  pinMode(2, INPUT);
//  pinMode(3, INPUT);
//  pinMode(5, INPUT);
//  pinMode(6, INPUT);
//  pinMode(7, INPUT);
//  pinMode(8, INPUT);
//  pinMode(9, INPUT);
//  pinMode(10, INPUT);
//  pinMode(11, INPUT);
//  pinMode(12, INPUT);
//  pinMode(13, INPUT);
//  pinMode(14, INPUT);
//  pinMode(15, INPUT);
}

// the loop function runs over and over again forever

void loop() {
  
  
  
  
  
  if (digitalRead(movepin) && !last_state){
    Serial.print(String("move STARED on ") + temp_send_counter + " second...\r\n"); 
    last_state = 1;
    String url = String("/objects/?object=")+sensor_id+"&op=set&p=moving&v=1";
    send_switch(url);
    
  }  
  if (!digitalRead(movepin) && last_state){
    Serial.print(String("move STOPPED on ") + temp_send_counter + " second...\r\n"); 
    last_state = 0;
    String url = String("/objects/?object=")+sensor_id+"&op=set&p=moving&v=0";
    send_switch(url);
  }
  
  
  
  
  
  
  
  
  
  
  
  currentTime = millis();  
  
if ((unsigned long)(currentTime - loop2Time) >= 60000) { 
  int h = dht.readHumidity();
  int t = dht.readTemperature();
  float freemem = system_get_free_heap_size();
 
  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t)) {
    Serial.println("Failed to read from DHT sensor!");
    return;
  }
  
//  Serial.println(t);
//  Serial.println(h);
//  Serial.println("----");
  String url = String("/objects/?script=tempsensor&name=")+sensor_id+"&temp="+t+"&hum="+h+"&freemem="+freemem;
  send_switch(url);

//Serial.println(digitalRead(0));
//Serial.println(digitalRead(1));
//Serial.println(digitalRead(2));
//Serial.println(digitalRead(3));
//Serial.println(digitalRead(4));
//Serial.println(digitalRead(5));
//Serial.println(digitalRead(6));
//Serial.println(digitalRead(7));
//Serial.println(digitalRead(8));
//Serial.println(digitalRead(9));
//Serial.println(digitalRead(10));
//Serial.println(digitalRead(11));
//Serial.println(digitalRead(12));
//Serial.println(digitalRead(13));
//Serial.println(digitalRead(14));
//Serial.println(digitalRead(15));
//Serial.println("--------------");

loop2Time = currentTime;
}
  
  // read the state of the switch into a local variable:
  int reading = digitalRead(buttonPin);

  if (reading != lastButtonState) {
    // reset the debouncing timer
    lastDebounceTime = millis();
  } 
  
  if ((millis() - lastDebounceTime) > debounceDelay) {

    if (reading == LOW && countStart && (millis() - btnDnTime) > long(holdTime))
        {
//        Serial.println("tringger down");  
        led2State = !led2State;
        String url = String("/objects/?script=holdbutton&name=")+sensor_id+"&type=remoteswitch&action="+led2State;
        send_switch(url);
        btnDnTime = millis();
        ignoreUp = 1;
        }
        
        
    if (reading != buttonState) {
      buttonState = reading;
      if (buttonState == LOW){
        countStart = 1;     
        btnDnTime = millis();   
      }
      

      // only toggle the LED if the new button state is HIGH
      if (buttonState == HIGH) {
       countStart = 0; 

       if (!ignoreUp){
//         if (led2State){
//             led2State = !led2State;
//             String url = String("/objects/?script=holdbutton&name=")+sensor_id+"&type=remoteswitch&action="+led2State;
//             send_switch(url);
//         }
//         else{
           ledState = !ledState;
           digitalWrite(ledPin, ledState);
           
           String url = String("/objects/?object=")+sensor_id+"&op=set&p=status&v="+ledState;
           send_switch(url);
//         }
         
       }
       ignoreUp = 0;
      }
    }
  }
  lastButtonState = reading;
  
  ///#######################################
  WiFiClient client = server.available();
  if (!client) {
    return;
  }
  
  // Wait until the client sends some data
//  Serial.println("new client");
  while(!client.available()){
    delay(1);
  }
  
  // Read the first line of the request
  String req = client.readStringUntil('\r');
  Serial.println(req);
  client.flush();
  
  // Match the request
  int val;
  if (req.indexOf("/?pin=off") != -1){
    val = 0;
    lastButtonState = 0;
    ledState = 0;
    digitalWrite(ledPin, val);
//    String url = String("/objects/?object=")+sensor_id+"&op=set&p=status&v="+ledState;
//    send_switch(url);
  }
  else if (req.indexOf("/?pin=on") != -1){
    val = 1;
    lastButtonState = 1;
    ledState = 1;
    digitalWrite(ledPin, val);
//    String url = String("/objects/?object=")+sensor_id+"&op=set&p=status&v="+ledState;
//    send_switch(url);
  }
    else if (req.indexOf("/?status") != -1){
    //fg
  }
  else {
    //Serial.println("invalid request");
    client.stop();
    return;
  }

  // Set GPIO2 according to the request
//  digitalWrite(ledPin, val);
  
  client.flush();

  // Prepare the response
  String s = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<!DOCTYPE HTML>\r\n<html>\r\n";
  s += ledState;
  s += "</html>\n";

  // Send the response to the client
  client.print(s);
  delay(1);
//  Serial.println("Client disonnected");
  
  //#########################
  
  
}


void send_switch(String turl){
  if (WiFi.status() == WL_CONNECTED){
      WiFiClient client;
      const int httpPort = 80;
      if (!client.connect(host, httpPort)) {
        Serial.println("connection failed");
        return;
      }
      
      //String url = String("/objects/?script=tempsensor&name=")+sensor_id+"&temp="+t+"&hum="+h+"&freemem="+freemem+"&moving="+moving_detected;
      String url = turl;
      Serial.print("Requesting URL: ");
      Serial.println(url);
      
      client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                   "Host: " + host + "\r\n" + 
                   "Connection: close\r\n\r\n");
      delay(50);
      
    //  while(client.available()){
    //    String line = client.readStringUntil('\r');
    ////    Serial.print(line);
    //  }
      
      Serial.println();
      Serial.println("closing connection");
      client.stop();
  }
  else{
    Serial.println("WIFI NOT CONNECTED");
  }

}
